/*
   Tamot --- Task Modelling Editor
   Copyright (C) 2000 Commonwealth Scientific and Industrial Research Organisation
   (CSIRO), Australia. All rights reserved. 

   The Software, owned by CSIRO Australia, has been developed by the CSIRO 
   Intelligent Interactive Technology group. 
   For more information, please see http://www.cmis.csiro.au/iit/.

   The Software is a beta-test version for internal research and evaluation purposes 
   by the Licensee. The Software is still at a development stage. 
   It has not undergone complete testing and may have inherent errors,
   bugs or deficiencies that can affect the operation of the Software. 

   We encourage your feedback and suggestions. Any bugs / suggestions found please email to
   Shijian.Lu@cmis.csiro.au and Thomas.Lo@cmis.csiro.au. 

   After you have signed the Beta-Test Software License Agreement with the CSIRO,
   CSIRO grants you a non-exclusive, non-transferable license to use it only for your 
   personal, non-commercial and lawful end use. Implied licenses are negated. 

   Warranty Disclaimer : 

   CSIRO supplies the Software on an "as is" basis and makes no warranties that use of
   the Software by the Licensee will not infringe any third partys 
   Intellectual Property Rights nor that the Software will be of merchantable quality,
   or suitable for a particular purpose. CSIRO excludes all terms, conditions and
   warranties implied by custom, the general law or statute in relation to the
   Software.
*/

//Source file: D:/Projects/Isolde/parser/Parser.java

//assumes the user has wordnet installed on their computer
//add comments at relevant areas

package parser.FSParser;

import java.util.*;
import domainModel.*;
import utility.*;
import taskModellingTool.*;
import javax.swing.tree.*;

import jwnl.data.*;
import jwnl.data.list.pointer_target.PointerTargetList;
import jwnl.data.list.pointer_target.PointerTargetTree;
import jwnl.data.relationship.*;
import jwnl.dictionary.DictionaryDatabase;
import jwnl.dictionary.hashtable.HashtableBackedDictionary;
import jwnl.dictionary.flatfile.FileBackedDictionary;
import java.util.Hashtable;
import java.io.RandomAccessFile;


/**
The parser can run any finite-state grammar similar to the ones shown in the current 
grammar files.  We chose a finite-state mechanism because it is simple and sufficient
for the task.  We will need grammars for the simple default case (3-word expressions -
agent-process-patient) as well as an action grammar (see action.grammar) and a state
grammar.

Notes: 
   - The code assumes that the "process" constituent is the verb and that all
     others are case-roll fillers of some sort.

   - The parser has to know about the guards on the transitions - these can't be 
     declarative.  We'll assume noun/verb/preposition/word.

   - The parser needs an accessor to return the constituents table numbers to the GUI 
     interface so it can set the heads and constituent boundaries properly.
*/
public class Parser {
  private Grammar _grammar;          // the finite-state grammar we are currently using
  public DmManager _dmManager;       // pointer to the domain model
  private Vector _constituentTables; // list of constituent tables
  //private Hashtable name2Index;      // maps from constituent name to constituent table index
  private String _cnlString;         // the string being parsed
  
  private GenericComponent _gc = null;   // shijian 20-4-2001
  private DictionaryDatabase db;     // access to wordnet. brasser 2001-6-6
  private ConstituentTable _currentTable;  //the constituent table currently being used //brasser 2001-7-12

  public Parser(DmManager inDmManager, String filename)
  {
    // Set up the grammar - parse() will initialize the grammar.
    _grammar = new Grammar(filename);
    _dmManager = inDmManager;
    db = new FileBackedDictionary(); //brasser 2001-6-6

    // create the list of constituent tables, initially with a single table //brasser 2001-7-12
    _constituentTables = new Vector();
    _currentTable = new ConstituentTable(_dmManager, _grammar);
    //_constituentTables.add(_currentTable);

  }


  //utility function for inserting a space in fromt of punctuation
  //used in parse function to add spaces in front of commas
  //code from java.sun.com
  //brasser 2001-7-19
  public static String replaceString (String oldString, String newString, String inputString){
    String outputString = "";
    int i;
    for (i=0; i<inputString.length(); i++) {
      if (!(inputString.regionMatches (true, i, oldString, 0, oldString.length())))
	outputString += inputString.charAt(i);
      else {
	outputString += newString;
	i += oldString.length()-1;
      }
    }
    return outputString;
  }


  /**
     Takes a string, parses it, and creates DM concepts/instances as appropriate.
     It returns a boolean indicating if the parse worked or not.

     It will include its own view (i.e., the parser interface).

     For now, it will "reparse" by clearing the previous parse and starting from scratch.
     It will set all the constituent starts to length(parseString).
     initialize the grammar
     tokenize the input string
     set currentPosition and workingBufferStart = 0
     gets list of guards, picks one, transitions the grammar - if none works,
     do an unknown transition.
     gets list of required operations and does them.
     @roseuid 395A007A02AB
  */
  public boolean parse(String cnlString)
  {
    //brasser 2001-7-19
    //organizes the string to be ready for use with String Tokenizer (spaces in front of punctuation, and all lowercase)
    //we may want to do the lowercase in a different place if we start using capitalization in some special way
    cnlString = replaceString(".\"", "\".", cnlString);
    cnlString = replaceString(",", " ,", cnlString);
    cnlString = replaceString(".", " .", cnlString);
    cnlString = replaceString("  ", " ", cnlString); //these can give us trouble
    //we may want to add spaces elsewhere in the future (such as at _"_ or _(_ )
    cnlString = cnlString.toLowerCase();
    _cnlString = cnlString;

    //    System.out.println("parsing: " + cnlString);
	

    // Some working variables for the parser
    int currentPosition;         // a cnlString index of the current parser position
    int workingBufferStart;      // an index to the beginning of the current constituent
    StringTokenizer tokenizedInput; // tokenized list of input words to parse
    Vector possibleTransitions;  // a vector of outgoing links from the current state
    String currentToken;         // the current word of the input
    String previousToken = null; // the previous word of the inpput
    String currentTransition;    // the chosen transition
    String nounReference;        // the name of the DM object instance found by isNoun
    String verbReference;        // the name of the DM concept instance found by isVerb
    String adjReference;         // the name of the DM ? instance found by isAdjective
    String pronReference;        // the name of the DM ? instance found by isPronoun
    String artReference;		 // the name of the DM ? instance found by isArticle
    Vector currentOperations;    // operations to perform as specified by the grammar
    String headReference;        // name of the DM thing referenced by the head
    String previousState;        // name of the previous state in the parse
    String previousState2;
    int headLength;              // number of words of the constituent head

    // Initialize the grammar, the constituent table, the context variables
    //     and the tokenized input.
    _grammar.initialize();
	
    _constituentTables.clear(); //brasser 2001-7-13
    _currentTable.setRole("action"); //brasser 2001-7-18
    for (int i = 0; i < _currentTable.getConstituents().size(); i++)   //changed here //brasser 2001-7-12
      	((Constituent)(_currentTable.getConstituents().elementAt(i))).initialize();
    currentPosition = 0;
    workingBufferStart = 0;
    if (cnlString.equals(""))
      	return _grammar.inTerminalState();
    tokenizedInput = new StringTokenizer(cnlString);
    
    //2001-8-13
    //shijian
    // mark the second word as "verb", if it is not
    // recognized word and marked the third word as "noun"
    // if it is not recognised
 	boolean thereIsNoun= false;
 	boolean thereIsVerb= false; 
 	
 	// Loop through the words of the input to drive the grammar.
    while (tokenizedInput.hasMoreTokens()) {
      	currentToken = tokenizedInput.nextToken();
      	possibleTransitions = _grammar.possibleTransitions();
      	currentTransition = "unknown";   // default
      	headReference = "";              // default
      	headLength = 0;                  // default
      	//System.out.println("   current token: |" + currentToken + "|");

      	// determine the POS of the input (there could be more than one of them I suppose)
      	//check if the token is punctuation -- jwnl doesn't handle the lookup well
      	//brasser 2001-7-
      	if (!isPunctuation(currentToken)) {
	      	nounReference = isNoun(currentToken, tokenizedInput);
	      	verbReference = isVerb(currentToken);
			adjReference = isAdjective(currentToken);
			pronReference = isPronoun(currentToken);
			artReference = isArticle(currentToken);
      	}
      	else {
			nounReference = "";
		    verbReference = "";
			adjReference = "";
			pronReference = "";
			artReference = "";
      	}
	  
	  //shijian 2001-8-13
	  if (!(nounReference.equals(""))) thereIsNoun = true;
	  if (!(verbReference.equals(""))) {
	  	thereIsVerb = true;
	  	thereIsNoun = false;
	  }
	  if (nounReference.equals("") && verbReference.equals("") &&
	  		adjReference.equals("") && pronReference.equals("")){
	  		if(!thereIsVerb && !thereIsNoun) {
      			nounReference = currentToken;
      			thereIsNoun = true;
	  		//	System.out.println(ii+ " noun: " +currentToken);

	  		} else if(thereIsNoun && !thereIsVerb) {
		  		verbReference =currentToken;
		  		thereIsVerb = true;
		  		//thereIsNoun = false;
	  		//	System.out.println(ii+ " verb: " +currentToken);
  			} else {	//if (!thereIsNoun && thereIsVerb){
      			nounReference = currentToken;
      			thereIsNoun = true;
	  		//	System.out.println(ii+ " noun: " +currentToken);
  			}
	  			
	  }
	  
	  // pick a transition that matches the POS and take it
	for (int i = 0; i < possibleTransitions.size(); i++) {	
		//shijian, 23-10-2001
		System.out.println("transition guard = "+
			possibleTransitions.elementAt(i));
		// Check for a literal word match (e.g., "at", "from")
		if (literalTransitionMatch(currentToken,
					   ((String)(possibleTransitions.elementAt(i))))) {
		  	currentTransition = ((String)(possibleTransitions.elementAt(i)));
		  	break;
		}
		// Check for an article match
		else if ((possibleTransitions.elementAt(i).equals("article")) &&
			(artReference != "")) {
		  	currentTransition = "article";
		  	break;
		}
		// Check for a verb/process match
		else if ((possibleTransitions.elementAt(i).equals("verb")) &&
			(verbReference != "")) {
		  	currentTransition = "verb";
		  	headReference = verbReference;
		  	headLength = currentToken.length(); //brasser 2001-6-22
		  	//headLength = verbReference.length();
		  	break;
		}
		// Check for a noun match (agent or object)
		else if ((possibleTransitions.elementAt(i).equals("noun")) &&
			(nounReference != "")) {
		  	currentTransition = "noun";
		  	headReference = nounReference;
		  	headLength = currentToken.length(); //brasser 2001-6-22
		  	//headLength = nounReference.length();
		  	break;
		}
		// Check for an adjective match
		else if ((possibleTransitions.elementAt(i).equals("adjective")) &&
			 (adjReference != "")) {
		  	currentTransition = "adjective";
		  	break;
		}
		// Check for a pronoun match
		else if ((possibleTransitions.elementAt(i).equals("pronoun")) &&
			 (pronReference != "")) {
		  	currentTransition = "pronoun";
		  	break;
		}
		// Check for simple word guards (for simple grammars that don't care about POS)
		else if (possibleTransitions.elementAt(i).equals("word")) {
		  	currentTransition = "word";
				  
		  	//shijian 8-2-2001
		  	// mark head of constituents for simple grammar
		  	headReference = currentToken;
		  	headLength = currentToken.length();
		  	break;
		}
	}

  	//System.out.println("      perform transition: " + currentTransition);
  	//previousState = _grammar.getCurrentState();  // save this for one of the operations
 	previousState = _grammar.getCurrentConstituent();
  	previousState2 = _grammar.getCurrentState();
  	currentOperations = _grammar.transition(currentTransition);
  	      System.out.println("transition: " + currentTransition);
  	      System.out.println("constituent: " + _grammar.getCurrentConstituent());
  	      System.out.println("previous state: " + previousState);
  	      System.out.println("current state: " + _grammar.getCurrentState() + "\n");
  	printConstituentTable();
  	System.out.println("\n\n");

  	// perform the operations specified by the grammar
	for (int i = 0; i < currentOperations.size(); i++) //change here
		switch (((Integer)(currentOperations.elementAt(i))).intValue())
	  	{
	  		case Transition.INCREMENT_CURRENT:
			    currentPosition += currentToken.length() + 1;
			    break;
			case Transition.START_EQ_CURRENT:
			    //((Constituent)(_constituents.elementAt(((Integer)(name2Index.get(_grammar.getCurrentState()))).intValue()))).setStartPosition(currentPosition);
			    //System.out.println("START_EQ_CURRENT: " + _grammar.getCurrentConstituent() + currentPosition + "\n");
			    ((Constituent)(_currentTable.getConstituents().
			    	elementAt(_currentTable.name2Index(_grammar.
			    	getCurrentConstituent())))).setStartPosition(currentPosition);
			    break;
			case Transition.START_EQ_WORKING:
			    //System.out.println("START_EQ_WORKING: " + _grammar.getCurrentConstituent() + workingBufferStart + "\n");
			    ((Constituent)(_currentTable.getConstituents().
			    	elementAt(_currentTable.name2Index(_grammar.
			    	getCurrentConstituent())))).setStartPosition(workingBufferStart);
			    break;
			case Transition.WORKING_EQ_START:
			    workingBufferStart = ((Constituent)(_currentTable.
			    	getConstituents().elementAt(_currentTable.name2Index(_grammar.
			    	getCurrentConstituent())))).startPosition();
			    break;
			case Transition.END_EQ_CURRENT_MINUS_1:
			    ((Constituent)(_currentTable.getConstituents().
			    	elementAt(_currentTable.name2Index(previousState)))).
			    	setEndPosition(currentPosition - 1);
			    break;
			case Transition.HEAD_EQ_CURRENT:
			    ((Constituent)(_currentTable.getConstituents().
			    	elementAt(_currentTable.name2Index(_grammar.
			    	getCurrentConstituent())))).setHeadPosition(currentPosition);
			    ((Constituent)(_currentTable.getConstituents().
			    	elementAt(_currentTable.name2Index(_grammar.
			    	getCurrentConstituent())))).setHeadLength(headLength);
			    ((Constituent)(_currentTable.getConstituents().
			    	elementAt(_currentTable.name2Index(_grammar.
			    	getCurrentConstituent())))).setDomainObjectName(headReference);
			    break;
			case Transition.ADD_TO_HEAD:
			    changeHead(_grammar.getCurrentConstituent(), 
			    	new Integer(((Constituent)(_currentTable.getConstituents().
			    	elementAt(_currentTable.name2Index(_grammar.getCurrentConstituent())))).
			    	startPosition()), new Integer(currentPosition-1));
			    break;
			case Transition.RESTART_PARSER:
			    //	    System.out.println("Restarting parser..." + _cnlString);
			    //	    System.out.println("currentPosition: " + currentPosition);
			    _constituentTables.add(_currentTable);
			    //brasser 2001-7-18
			    //this variable should probably be renamed to previousState, and previousState to previous constituent //look here
			    if (previousState2.equals("period"))
			      _constituentTables.add(".");
			    _currentTable = new ConstituentTable(_dmManager, _grammar);
			    for (int j = 0; j < _currentTable.getConstituents().size(); j++)
			      ((Constituent)(_currentTable.getConstituents().elementAt(j))).initialize();
			    break;
			 case Transition.SET_ROLE:
			    //	    System.out.println("role: " + previousState);
			    _currentTable.setRole(previousState);
			    break;
	  }
	  

      //brasser 2001-7-17
      //this is a repetition of the code above. It is here to take care of epsilon moves
      //we don't want to get the next token before performing an epsilon move, so we do it here
      possibleTransitions = _grammar.possibleTransitions();
      for (int k = 0; k < possibleTransitions.size(); k++) {
	// Check for an epsilon move
	if (possibleTransitions.elementAt(k).equals("E")) {
	  currentTransition = "E";
	  headReference = currentToken;
	  headLength = currentToken.length();
			  
	  previousState = _grammar.getCurrentState();
	  currentOperations = _grammar.transition(currentTransition);
	  //System.out.println("transition: " + currentTransition);
	  //System.out.println("constituent: " + _grammar.getCurrentConstituent());
	  //System.out.println("previous state: " + previousState);
	  //System.out.println("current state: " + _grammar.getCurrentState() + "\n");
	  //printConstituentTable();
	  //System.out.println("\n\n");

	  // perform the operations specified by the grammar
	  for (int i = 0; i < currentOperations.size(); i++) //change here
	    switch (((Integer)(currentOperations.elementAt(i))).intValue())
	      {
	      case Transition.INCREMENT_CURRENT:
		currentPosition += currentToken.length() + 1;
		break;
	      case Transition.START_EQ_CURRENT:
	    			//((Constituent)(_constituents.elementAt(((Integer)(name2Index.get(_grammar.getCurrentState()))).intValue()))).setStartPosition(currentPosition);
		//		System.out.println("START_EQ_CURRENT: " + _grammar.getCurrentConstituent() + currentPosition + "\n");
		((Constituent)(_currentTable.getConstituents().elementAt(_currentTable.name2Index(_grammar.getCurrentConstituent())))).setStartPosition(currentPosition);
		break;
	      case Transition.START_EQ_WORKING:
		((Constituent)(_currentTable.getConstituents().elementAt(_currentTable.name2Index(_grammar.getCurrentConstituent())))).setStartPosition(workingBufferStart);
		break;
	      case Transition.WORKING_EQ_START:
		workingBufferStart = ((Constituent)(_currentTable.getConstituents().elementAt(_currentTable.name2Index(_grammar.getCurrentConstituent())))).startPosition();
		break;
	      case Transition.END_EQ_CURRENT_MINUS_1:
		((Constituent)(_currentTable.getConstituents().elementAt(_currentTable.name2Index(previousState)))).setEndPosition(currentPosition - 1);
		break;
	      case Transition.HEAD_EQ_CURRENT:
		((Constituent)(_currentTable.getConstituents().elementAt(_currentTable.name2Index(_grammar.getCurrentConstituent())))).setHeadPosition(currentPosition);
		((Constituent)(_currentTable.getConstituents().elementAt(_currentTable.name2Index(_grammar.getCurrentConstituent())))).setHeadLength(headLength);
		((Constituent)(_currentTable.getConstituents().elementAt(_currentTable.name2Index(_grammar.getCurrentConstituent())))).setDomainObjectName(headReference);
		break;
	      case Transition.ADD_TO_HEAD:
	    			//headLength = currentPosition - ((Constituent)(_constituents.elementAt(((Integer)(name2Index.get(_grammar.getCurrentState()))).intValue()))).startPosition() - 1;
		//((Constituent)(_constituents.elementAt(((Integer)(name2Index.get(_grammar.getCurrentConstituent()))).intValue()))).setHeadLength(headLength);
		changeHead(_grammar.getCurrentConstituent(), new Integer(((Constituent)(_currentTable.getConstituents().elementAt(_currentTable.name2Index(_grammar.getCurrentConstituent())))).startPosition()), new Integer(currentPosition-1));
		//((Constituent)(_constituents.elementAt(((Integer)(name2Index.get(_grammar.getCurrentConstituent()))).intValue()))).setDomainObjectName(headReference);
		break;
	      case Transition.RESTART_PARSER:
		//		System.out.println("Restarting parser..." + _cnlString);
		_constituentTables.add(_currentTable);
		//		System.out.println("currentPosition: " + currentPosition);
		workingBufferStart = currentPosition; // gets rid of wrong premodifiers
		//brasser 2001-7-18
		if (previousState.equals("period"))
		  _constituentTables.add(".");
		_currentTable = new ConstituentTable(_dmManager, _grammar);
		for (int j = 0; j < _currentTable.getConstituents().size(); j++)
		  ((Constituent)(_currentTable.getConstituents().elementAt(j))).initialize();
		//((Constituent)(_constituents.elementAt(((Integer)(name2Index.get(_grammar.getCurrentConstituent()))).intValue()))).setEndPosition(currentPosition);
		//createDM(null);
		break;				
	      case Transition.SET_ROLE:
		//		System.out.println("role: " + previousState);
		_currentTable.setRole(previousState);
		break;
	      }
	}
      }
	  
    }

    //System.out.println("out of the loop in state: " + _grammar.getCurrentState());

    // fix one special case: the last sentence constituent will have a
    // startPosition and a -1 endPosition
    //make sure it is a constituent //brasser 2001-7-6
    if (_grammar.getCurrentConstituent() != null)  //changed here //brasser 2001-7-12 
      ((Constituent)(_currentTable.getConstituents().elementAt(_currentTable.name2Index(_grammar.getCurrentConstituent())))).setEndPosition(cnlString.length());
    _constituentTables.add(_currentTable); //brasser 2001-7-13
	
    // return the appropriate status
    return _grammar.inTerminalState();
  }

  /**
     Checks the list of known GUI and domain objects

     returns the name of the DM object that was recognized

     It should match the longest possible DM object (e.g., "task model" rather than
     just "task"). and it should return the length of that object name (using the
     length of the internal name is not good enough).
     @roseuid 395A021B0097
  */
	
  public String isNoun(String token, StringTokenizer st)
  {
    return isNoun(token);  // deal with multiple word entities later - kvlinden, 9jan2001
  }

  //added wordnet lookup. brasser 2001-6-6
  private String isNoun(String token)
  {
    IndexWord iw;
    iw = db.lookupBaseForm(POS.NOUN, token);
	
    // Check the domain model for the 3 "types" of objects.
    // We shouldn't have to do this, there should be one "object" parent concept.
    if ((_dmManager.getEConcept(token) != null)
	|| (_dmManager.getUserConcept(token) != null)
	|| (_dmManager.getInterfaceConcept(token) != null)
	|| (_dmManager.getSystemConcept(token) != null)
	)
      return token;
    else if (iw != null)
      return iw.getLemma();
    //return token;  
    else
      return "";
  }

  // brasser 2001-6-6
  private String isAdjective(String token)
  {
    IndexWord iw;
    iw = db.lookupBaseForm(POS.ADJECTIVE, token);
	
    if (iw != null)
      return token;  
    else
      return "";
  }
  
  // brasser 2001-6-6
  private String isPronoun(String token)
  {
    if ((token.equals("this")) || (token.equals("that")) || (token.equals("these"))
	|| (token.equals("those")) || (token.equals("his")) || (token.equals("her"))
	|| (token.equals("you")) || (token.equals("he")) || (token.equals("she"))
	|| (token.equals("it")) || (token.equals("s/he"))) 
      return token;  
    else
      return "";
  }

  /**
     checks the list of predefined GUI verbs.  We may also want to use a list of
     "general" verbs from wordnet or something like that.

     returns the length of the recognized string and a pointer to the concept for
     that process (if it exists).
     @roseuid 395A0243035C
  */
	
  //added wordnet lookup. brasser 2001-6-6
  public String isVerb(String token)
  {
    IndexWord iw;
    iw = db.lookupBaseForm(POS.VERB, token);
  
    if (_dmManager.getAConcept(token) != null)
      return token;
    else if (iw != null) 
      {
	return iw.getLemma();
	//return token;  
      }
    else
      return "";
  }
  
  //brasser 2001-7-6
  public String isArticle(String token)
  {
    if ((token.equals("a")) || (token.equals("an")) || (token.equals("the")))
      return token;  
    else
      return "";
  }
  
  //check to see if something is punctuation //brasser 2001-7-6
  public boolean isPunctuation(String token) 
  {
    if ((token.equals(".")) || (token.equals(","))) 
      return true; 
    else
      return false;
  }

  /**
     returns true if the current token matches a literal transition (surrounded by quotes
     in the grammar)
     @roseuid 395A024902EC
  */
  public boolean literalTransitionMatch(String token, String transition)
  {
    return (transition.startsWith("\"") &&
	    token.equals(transition.substring(1,transition.length() - 1)));
  }

  // a utility for debugging.
  public void printConstituentTable()
  {
    for (int i = 0; i < _currentTable.getConstituents().size(); i++) //changed here //brasser 2001-7-12
      ((Constituent)(_currentTable.getConstituents().elementAt(i))).print();
  }

  /**
     goes through the constituent start positions, creating DM concepts and instances
     as required.  It is careful not to "recreate" entities when they already exist in the DM.
     It is hard-coded for the constituents we know to be in the grammar.
     @roseuid 395A07B50266
  */
	
  //brasser 2001-7-13
  //make this function a wrapper class for the function in ConstituentTable
  public ActionInstance createDM(GenericComponent gc)
  {
    ActionInstance ai;  // a utility variable for entity instances
    //    System.out.println("Size: " + _constituentTables.size());
    //shijian, 22-10-2001
	ai = _currentTable.createDM(gc, _cnlString, _cnlString);
	return ai;
 /*   if (_constituentTables.size() < 2) 
      {
		ai = _currentTable.createDM(gc, _cnlString, _cnlString);
    	return ai;
      }
    else 
      {
		System.out.println("*WARNING*");
		return null;
     }
  */  
    
  } 
  
  //brasser 2001-7-13
  //creates task and domain model information
  public void createTM(GenericComponent gc) 
  {
    ConstituentTable _otherTable = null;
    String previousTask = null;
    String prevHighTask = null;
    boolean pass = false;
    Vector temp = new Vector();
    ActionInstance ai;        // a utility variable
  	
    /*System.out.println("**************************************************");
      for (int i = 0; i < _constituentTables.size(); i++) 
      {
      System.out.println(_constituentTables.elementAt(i));
      }
      System.out.println("**************************************************");*/
	
    for (int i = 0; i < _constituentTables.size(); i++) 
      {
	if (((_constituentTables.elementAt(i)).getClass()).isAssignableFrom(_currentTable.getClass()))
	  {
	    _currentTable = (ConstituentTable)_constituentTables.elementAt(i);
	    //	    System.out.println(_currentTable.getRole());
	    pass = false;
			
	    //to take care of blank ones at the end
	    if(((Constituent)(_currentTable.getConstituents().elementAt(_currentTable.name2Index("process")))).startPosition()==-1)
	      pass=true;
	  }
	else
	  pass = true;
		
	//System.out.println(((ConstituentTable)_constituentTables.elementAt(i)).getRole());
		
	//take care of a purpose clause in the first sentence - will be purpose for whole paragraph
	if (i == 0) //eventually we may want to add more guards, especially if it is for a more structured text
	  {
		
	    while ((i < (_constituentTables.size()-1)) && (((_constituentTables.elementAt(i)).getClass()).isAssignableFrom(_currentTable.getClass()))) //go until period or end of vector
	      {
		_currentTable = (ConstituentTable)_constituentTables.elementAt(i);
		if (_currentTable.getRole() == ConstituentTable.PURPOSE_CLAUSE) 
		  {
		    Task t = NewManager.newTask(-1,-1, createName(_currentTable), true);
		    setStyle(t, _currentTable); //brasser 2001-8-17
		    ai = _currentTable.createDM(gc, _cnlString, createName(_currentTable));
		    t.setSemantics(ai.getID());
				
		  }
		else
		  {
		    temp.add(_currentTable);
		  }
		i++;
				
	      }
	    if (temp.size() < 1)         //brasser 2001-8-17. fixes a bug for sentences with no period at the end
	      temp.add(_currentTable);
				
	    _constituentTables.addAll(i+1, temp); //brasser 2001-7-27
	    _constituentTables.add(".");
	  }
		
	//depending on the type of clause, we need to string the tasks together differently
	else if (pass)
	  ; //do nothing for punctuation
	else if (_currentTable.getRole() == ConstituentTable.PURPOSE_CLAUSE)
	  {
	    Task t = NewManager.newTask(-1,-1,createName(_currentTable));
	    setStyle(t, _currentTable); //brasser 2001-8-17
	    ai = _currentTable.createDM(gc, _cnlString, createName(_currentTable));	
	    t.setSemantics(ai.getID());
	    if (prevHighTask != null)			
	      NewManager.newLink(prevHighTask,createName(_currentTable));
	    while ((i < (_constituentTables.size()-1)) && (((_constituentTables.elementAt(i+1)).getClass()).isAssignableFrom(_currentTable.getClass()))) //go until period or end of vector
	      {
		_otherTable = (ConstituentTable)_constituentTables.elementAt(i+1);
		Task t2 = NewManager.newTask(t.getName(),createName(_otherTable), _cnlString, true);
		setStyle(t2, _otherTable); //brasser 2001-8-17
		if (previousTask != null)			
		  NewManager.newLink(t.getName(), previousTask,t2.getName());
		//NewManager.newLink(t.getName(), previousTask,createName(_otherTable));
		previousTask = t2.getName();
		ai =_otherTable.createDM(gc, _cnlString, createName(_otherTable));
		t2.setSemantics(ai.getID());
				
		i++;
	      }
	    prevHighTask = t.getName();
	  }
	else if (_currentTable.getRole() == ConstituentTable.PRECONDITION)
	  {
	    if ((i < (_constituentTables.size()-1)) && (((_constituentTables.elementAt(i+1)).getClass()).isAssignableFrom(_currentTable.getClass()))) 
	      {
		_otherTable = (ConstituentTable)_constituentTables.elementAt(i+1);
		Task t = NewManager.newTask(-1,-1,createName(_otherTable));
		setStyle(t, _otherTable); //brasser 2001-8-17
		t.setPrecon(createName(_currentTable));
		if (prevHighTask != null)			
		  NewManager.newLink(prevHighTask,t.getName());
		//NewManager.newLink(prevHighTask,createName(_otherTable));
		prevHighTask = t.getName();
		ai = _otherTable.createDM(gc, _cnlString, createName(_otherTable));
		t.setSemantics(ai.getID());
		i++;
	      }
			
	  }
	else if (_currentTable.getRole() == ConstituentTable.ACTION_CLAUSE) 
	  {
			
	    if (i < (_constituentTables.size()-1)) { //check to see there is something in the next spot
	      if (((_constituentTables.elementAt(i+1)).getClass()).isAssignableFrom(_currentTable.getClass())) //check to make sure it isn't punctuation...
		{
		  _otherTable = (ConstituentTable)_constituentTables.elementAt(i+1);
		  if (_otherTable.getRole() == ConstituentTable.PURPOSE_CLAUSE) //regular clause followed by purpose clause 
		    {
		      //create the parent task
		      Task t = NewManager.newTask(-1,-1,createName(_otherTable));
		      setStyle(t, _otherTable); //brasser 2001-8-17
		      ai = _otherTable.createDM(gc, _cnlString, createName(_otherTable));
		      t.setSemantics(ai.getID());
		      if (prevHighTask != null)			
			NewManager.newLink(prevHighTask,t.getName());
		      prevHighTask = t.getName();	
		      //create the child task with the correct parent
		      Task t2 = NewManager.newTask(t.getName(),createName(_currentTable), _cnlString, true);
		      setStyle(t2, _currentTable); //brasser 2001-8-17
		      ai = _currentTable.createDM(gc, _cnlString, createName(_currentTable));
		      t2.setSemantics(ai.getID());
		      //linking???
		      //need to add onto i because we have already created the next object
		      i++;
		    }
		  else if (_otherTable.getRole() == ConstituentTable.PRECONDITION) 
		    {
		      Task t = NewManager.newTask(-1,-1,createName(_currentTable));
		      setStyle(t, _currentTable); //brasser 2001-8-17
		      t.setPrecon(createName(_otherTable));
		      if (prevHighTask != null)			
			NewManager.newLink(prevHighTask,t.getName());
		      prevHighTask = t.getName();	
		      ai = _currentTable.createDM(gc, _cnlString, createName(_currentTable));
		      t.setSemantics(ai.getID());
		      i++;	
		    }
		  else if (_otherTable.getRole() == ConstituentTable.IMPLEMENT_CLAUSE) //HAVEN'T TESTED YET 
		    {
		      //create the parent task
		      Task t = NewManager.newTask(-1,-1,createName(_currentTable));
		      setStyle(t, _currentTable); //brasser 2001-8-17
		      ai = _currentTable.createDM(gc, _cnlString, createName(_currentTable));
		      t.setSemantics(ai.getID());
		      if (prevHighTask != null)			
			NewManager.newLink(prevHighTask,t.getName());
		      prevHighTask = t.getName();	
		      //create the child task with the correct parent
		      Task t2 = NewManager.newTask(t.getName(),createName(_otherTable), _cnlString, true);
		      setStyle(t2, _otherTable); //brasser 2001-8-17
		      ai = _otherTable.createDM(gc, _cnlString, createName(_otherTable));
		      t2.setSemantics(ai.getID());
		      //linking???
		      //need to add onto i because we have already created the next object
		      i++;	
		    }
		  else //if next is not a purpose clause or precondition...regular clause
		    {
		      Task t = NewManager.newTask(-1,-1,createName(_currentTable));
		      setStyle(t, _currentTable); //brasser 2001-8-17
		      if (prevHighTask != null)			
			NewManager.newLink(prevHighTask,t.getName());
		      prevHighTask = t.getName();	
		      ai = _currentTable.createDM(gc, _cnlString, createName(_currentTable));	
		      t.setSemantics(ai.getID());
		    }
		}
	      else //if next is punctuation...regular clause
		{
		  Task t = NewManager.newTask(-1,-1,createName(_currentTable));
		  setStyle(t, _currentTable); //brasser 2001-8-17
		  if (prevHighTask != null)			
		    NewManager.newLink(prevHighTask,t.getName());
		  prevHighTask = t.getName();
		  ai =_currentTable.createDM(gc, _cnlString, createName(_currentTable));
		  t.setSemantics(ai.getID()); 
		  //increment i to skip over the punctuation
		  i++;
		}
				
	    }
	    else //regular clause
	      {
		Task t = NewManager.newTask(-1,-1,createName(_currentTable));
		setStyle(t, _currentTable); //brasser 2001-8-17
		if (prevHighTask != null)			
		  NewManager.newLink(prevHighTask,t.getName());
		prevHighTask = t.getName();
		ai = _currentTable.createDM(gc, _cnlString, createName(_currentTable));	
		t.setSemantics(ai.getID());
	      }

	  }
		
      }
  }
  
  
  private String createName(ConstituentTable inTable) 
  {
    //get the seperate pieces, and then we can create our own custom tasks
    //"after clicking the button" --> "user click button"
    String process = "";
    String actor = "";
    String actee = "";
    String instrument = "";
    String location = "";
    String source = "";
    String destination = "";
    String spatialLocation = "";
    String rolePlaying = "";
    Constituent c = ((Constituent)(inTable.getConstituents().elementAt(inTable.name2Index("process"))));
    process = c.domainObjectName() + _cnlString.substring(c.headPosition() + c.headLength(), c.endPosition()) + " ";
    c = ((Constituent)(inTable.getConstituents().elementAt(inTable.name2Index("actor"))));
    if (c.startPosition() != -1)
      actor = _cnlString.substring(c.startPosition(), c.endPosition()) + " ";
    c = ((Constituent)(inTable.getConstituents().elementAt(inTable.name2Index("actee"))));
    if (c.startPosition() != -1)
      actee = _cnlString.substring(c.startPosition(), c.endPosition()) + " ";
    c = ((Constituent)(inTable.getConstituents().elementAt(inTable.name2Index("instrument"))));
    if (c.startPosition() != -1)
      instrument = "using " + _cnlString.substring(c.startPosition(), c.endPosition()) + " ";	
    c = ((Constituent)(inTable.getConstituents().elementAt(inTable.name2Index("location"))));
    if (c.startPosition() != -1)
      location = "at " + _cnlString.substring(c.startPosition(), c.endPosition()) + " ";
    c = ((Constituent)(inTable.getConstituents().elementAt(inTable.name2Index("source"))));
    if (c.startPosition() != -1)
      source = "from " + _cnlString.substring(c.startPosition(), c.endPosition()) + " ";
    c = ((Constituent)(inTable.getConstituents().elementAt(inTable.name2Index("destination"))));
    if (c.startPosition() != -1)
      destination = "to " + _cnlString.substring(c.startPosition(), c.endPosition()) + " ";	
    c = ((Constituent)(inTable.getConstituents().elementAt(inTable.name2Index("spatial-location"))));
    if (c.startPosition() != -1)
      spatialLocation = "on " + _cnlString.substring(c.startPosition(), c.endPosition()) + " ";	
    c = ((Constituent)(inTable.getConstituents().elementAt(inTable.name2Index("role-playing"))));
    if (c.startPosition() != -1)
      rolePlaying = "as " + _cnlString.substring(c.startPosition(), c.endPosition()) + " ";	

	
    //System.out.println("actor: " + actor);
    //System.out.println("process: " + process);
    //System.out.println("actee: " + actee);
	
	
    String newName = actor + process + actee + rolePlaying + location + spatialLocation + source + destination + instrument;
    return newName.trim(); //get rid of white space at end
  }  

  /**
     This function will assume that it will never be asked to move a start forward or
     backward beyond the next start in the default consituent order.

     The constituent will either be agent/process/patient/prepositional phrase.
     @roseuid 395B42DC03C8
  */
  public boolean moveConstituent(String constituentName, Integer newStartPosition, Integer newEndPosition)
  { //change here
    ((Constituent)(_currentTable.getConstituents().elementAt(_currentTable.name2Index(constituentName)))).setStartPosition(newStartPosition.intValue());
    ((Constituent)(_currentTable.getConstituents().elementAt(_currentTable.name2Index(constituentName)))).setEndPosition(newEndPosition.intValue());
    return false;
  }

  /**
     Changes the head of the passed constituent.
     @roseuid 395B49E3004A
  */
  public boolean changeHead(String constituentName, Integer newHeadStart, Integer newHeadEnd)
  {
  	System.out.println("head start="+newHeadStart.intValue());
  	System.out.println("head end="+newHeadEnd.intValue());

    ((Constituent)(_currentTable.getConstituents().
    	elementAt(_currentTable.name2Index(constituentName)))).
    	setHeadPosition(newHeadStart.intValue());
    ((Constituent)(_currentTable.getConstituents().
    	elementAt(_currentTable.name2Index(constituentName)))).
    	setHeadLength(newHeadEnd.intValue() - newHeadStart.intValue());
    ((Constituent)(_currentTable.getConstituents().
    	elementAt(_currentTable.name2Index(constituentName)))).
    	setDomainObjectName(_cnlString.substring(newHeadStart.intValue(),
		newHeadEnd.intValue()));
    return false;
  }

  public String cnlString()
  {
    return _cnlString;
  }
  //change here //brasser 2001-7
  public Vector constituents()
  {
    return _currentTable.getConstituents();
  }

  public Vector possibleTransitions()
  {
    return _grammar.possibleTransitions();
  }
  
  //brasser 2001-8-13
  public Vector allGuards() 
  {
    Vector v = new Vector();
    v.add("noun");
    v.add("verb");
    v.add("adjective");
    v.add("article");
    v.add("pronoun");
	
    return v;
  }
  
  //brasser 2001-8-13
  public Vector allTransitions() 
  {
    Vector v = new Vector();
    v.add("INCREMENT_CURRENT");
    v.add("START_EQ_CURRENT");
    v.add("START_EQ_WORKING");
    v.add("WORKING_EQ_START");
    v.add("HEAD_EQ_CURRENT");
    v.add("END_EQ_CURRENT_MINUS_1");
    v.add("ADD_TO_HEAD");
    v.add("RESTART_PARSER");
    v.add("SET_ROLE");
	
    return v;
  }
  
  //brasser 2001-8-17
  private void setStyle(Task t, ConstituentTable table) 
  {
    Constituent c = ((Constituent)(table.getConstituents().elementAt(table.name2Index("actor"))));
    if(c.headLength() > 0 && _cnlString.substring(c.headPosition(), c.endPosition()).equals("system"))
      t.setStyle("Automatic");
	
  	
  }
}